import typing

import requests

from .model import Resource
from .consent import ConsentRequest, ConsentSession
from .login import LoginRequest, LoginSession
from .logout import LogoutRequest
from .oauth2 import OAuth2Client
from .version import Version


class HydraAdmin(Resource):
    def __init__(self, url: str, session: requests.Session = None):
        self.url_ = url
        self.session_ = session or requests.Session()

    def clients(
        self, limit: int = None, offset: int = None
    ) -> typing.List[OAuth2Client]:
        return OAuth2Client._list(self, limit, offset)

    def client(self, id: str) -> OAuth2Client:
        return OAuth2Client._get(self, id)

    def create_client(
        self,
        allowed_cors_origins: typing.List[str] = None,
        audience: typing.List[str] = None,
        backchannel_logout_session_required: bool = None,
        backchannel_logout_uri: str = None,
        client_id: str = None,
        client_name: str = None,
        client_secret: str = None,
        client_secret_expires_at: int = None,
        client_uri: str = None,
        contacts: typing.List[str] = None,
        frontchannel_logout_session_required: bool = None,
        frontchannel_logout_uri: str = None,
        grant_types: typing.List[str] = None,
        jwks: dict = None,
        jwks_uri: str = None,
        logo_uri: str = None,
        owner: str = None,
        policy_uri: str = None,
        post_logout_redirect_uris: typing.List[str] = None,
        redirect_uris: typing.List[str] = None,
        redirect_object_signing_alg: str = None,
        request_uris: typing.List[str] = None,
        response_uris: typing.List[str] = None,
        scope: str = None,
        sector_identifier_uri: str = None,
        subject_type: str = None,
        token_endpoint_auth_method: str = None,
        tos_uri: str = None,
        userinfo_signed_response_alg: str = None,
    ) -> OAuth2Client:
        return OAuth2Client.create(
            self,
            allowed_cors_origins,
            audience,
            backchannel_logout_session_required,
            backchannel_logout_uri,
            client_id,
            client_name,
            client_secret,
            client_secret_expires_at,
            client_uri,
            contacts,
            frontchannel_logout_session_required,
            frontchannel_logout_uri,
            grant_types,
            jwks,
            jwks_uri,
            logo_uri,
            owner,
            policy_uri,
            post_logout_redirect_uris,
            redirect_uris,
            redirect_object_signing_alg,
            request_uris,
            response_uris,
            scope,
            sector_identifier_uri,
            subject_type,
            token_endpoint_auth_method,
            tos_uri,
            userinfo_signed_response_alg,
        )

    def login_request(self, challenge: str) -> LoginRequest:
        return LoginRequest._get(self, challenge)

    def consent_request(self, challenge: str) -> ConsentRequest:
        return ConsentRequest._get(self, challenge)

    def logout_request(self, challenge: str) -> LogoutRequest:
        return LogoutRequest._get(self, challenge)

    def consent_sessions(self, subject: str) -> typing.Iterator[ConsentSession]:
        yield from ConsentSession._list(self, subject)

    def revoke_consent_sessions(self, subject: str, client: str = None) -> None:
        ConsentSession._revoke(self, subject, client)

    def invalidate_login_sessions(self, subject: str) -> None:
        LoginSession._invalidate_all(self, subject)

    def version(self) -> str:
        return Version._get(self)



from __future__ import annotations

import typing

import attr

from . import exceptions
from .model import Resource
from .utils import urljoin

if typing.TYPE_CHECKING:
    from .api import HydraAdmin


@attr.s(auto_attribs=True, kw_only=True)
class Version(Resource):

    url_ = "/version"

    @classmethod
    def _get(cls, api: HydraAdmin) -> str:
        url = urljoin(api.url_, cls.url_)
        response = api._request("GET", url)
        payload = response.json()
        return payload["version"]



from __future__ import annotations

import typing

import attr

from .common import OpenIDConnectContext
from .model import Resource
from .oauth2 import OAuth2Client
from .utils import filter_none, urljoin

if typing.TYPE_CHECKING:
    from .api import HydraAdmin


@attr.s(auto_attribs=True, kw_only=True)
class LoginRequest(Resource):
    challenge: str
    client: OAuth2Client
    oidc_context: OpenIDConnectContext
    request_url: str
    requested_access_token_audience: typing.List[str]
    requested_scope: typing.List[str]
    session_id: str
    skip: bool
    subject: str
    url_ = "/oauth2/auth/requests/login"

    def _post_bind(self):
        self.url_ = urljoin(self.parent_.url_, self.url_)

    @classmethod
    def _params(cls, challenge: str) -> dict:
        return {"login_challenge": challenge}

    @classmethod
    def _get(cls, api: HydraAdmin, challenge: str) -> LoginRequest:
        url = urljoin(api.url_, cls.url_)
        response = api._request("GET", url, cls._params(challenge))
        return cls._from_dict(response.json(), parent=api)

    def accept(
        self,
        subject: str,
        acr: str = None,
        context: dict = None,
        force_subject_identifier: str = None,
        remember: bool = False,
        remember_for: int = None,
    ) -> str:
        data = filter_none(
            {
                "acr": acr,
                "context": context,
                "force_subject_identifier": force_subject_identifier,
                "remember": remember,
                "remember_for": remember_for,
                "subject": subject,
            }
        )
        url = urljoin(self.url_, "accept")
        response = self._request(
            "PUT", url, params=self._params(self.challenge), json=data
        )
        payload = response.json()
        return payload["redirect_to"]

    def reject(
        self,
        error: str = None,
        error_debug: str = None,
        error_description: str = None,
        error_hint: str = None,
        status_code: int = None,
    ) -> str:
        url = urljoin(self.url_, "reject")
        data = filter_none(
            {
                "error": error,
                "error_debug": error_debug,
                "error_description": error_description,
                "error_hint": error_hint,
                "status_code": status_code,
            }
        )
        response = self._request(
            "PUT", url, params=self._params(self.challenge), json=data
        )
        payload = response.json()
        return payload["redirect_to"]


@attr.s(auto_attribs=True, kw_only=True)
class LoginSession(Resource):

    url_ = "/oauth2/auth/sessions/login"

    def _post_bind(self):
        self.url_ = urljoin(self.parent_.url_, self.url_)

    @classmethod
    def _params(cls, subject: str) -> dict:
        return {"subject": subject}

    @classmethod
    def _invalidate_all(cls, api: HydraAdmin, subject: str) -> None:
        url = urljoin(api.url_, cls.url_)
        # This returns 204/201 without any content
        api._request("DELETE", url, params=cls._params(subject))



import typing

import attr

from .model import Entity


@attr.s(auto_attribs=True, kw_only=True)
class OpenIDConnectContext(Entity):
    acr_values: typing.List[str]
    display: str
    id_token_hint_claims: dict
    login_hint: str
    ui_locales: typing.List[str]



def filter_none(data: dict) -> dict:
    return {k: v for k, v in data.items() if v is not None}


def urljoin(url: str, *parts: str) -> str:
    return "/".join(
        (url.rstrip("/"), *(p.strip("/") for p in parts[:-1]), (parts[-1]).lstrip("/"))
    )



class HydraException(Exception):
    pass


class UnboundResourceError(HydraException):
    pass


class TransportError(HydraException):
    pass


class ConnectionError(TransportError):
    pass


class HTTPError(HydraException):
    pass


class BadRequest(HTTPError):
    pass


class Unauthorized(HTTPError):
    pass


class Forbidden(HTTPError):
    pass


class NotFound(HTTPError):
    pass


class ServerError(HTTPError):
    pass


status_map = {
    400: BadRequest,
    401: Unauthorized,
    403: Forbidden,
    404: NotFound,
    500: ServerError,
}



from __future__ import annotations

from datetime import datetime
import typing

import attr
import dateutil.parser

from .model import Entity, Resource
from .utils import filter_none, urljoin

if typing.TYPE_CHECKING:
    from .api import HydraAdmin


@attr.s(auto_attribs=True, kw_only=True)
class JSONWebKey(Entity):
    alg: str
    crv: str
    d: str
    dp: str
    dq: str
    e: str
    k: str
    kid: str
    kty: str
    n: str
    p: str
    q: str
    qi: str
    use: str
    x: str
    x5c: str
    y: str


@attr.s(auto_attribs=True, kw_only=True)
class JSONWebKeySet(Entity):
    keys = typing.List[JSONWebKey]


@attr.s(auto_attribs=True, kw_only=True)
class OAuth2Client(Resource):
    allowed_cors_origins: typing.List[str]
    audience: typing.List[str]
    backchannel_logout_session_required: bool = False
    backchannel_logout_uri: str = ""
    client_id: str
    client_name: str
    client_secret: typing.Optional[str] = None
    client_secret_expires_at: datetime = attr.ib(
        converter=datetime.fromtimestamp  # type: ignore
    )
    client_uri: str
    contacts: typing.List[str]
    frontchannel_logout_session_required: bool = False
    frontchannel_logout_uri: str = ""
    grant_types: typing.List[str]
    jwks: typing.Optional[JSONWebKeySet] = None
    jwks_uri: str = ""
    logo_uri: str
    owner: str
    policy_uri: str
    post_logout_redirect_uris: typing.List[str] = attr.ib(factory=list)
    redirect_uris: typing.List[str] = attr.ib(factory=list)
    request_object_signing_alg: typing.Optional[str] = None
    request_uris: typing.List[str] = attr.ib(factory=list)
    response_types: typing.List[str]
    scope: str
    sector_identifier_uri: typing.Optional[str] = None
    subject_type: str
    token_endpoint_auth_method: str
    tos_uri: str
    updated_at: datetime = attr.ib(converter=dateutil.parser.parse)
    userinfo_signed_response_alg: str

    url_ = "/clients"

    def _post_bind(self):
        self.url_ = urljoin(self.parent_.url_, self.url_, self.client_id)

    @classmethod
    def _list(
        cls, api: HydraAdmin, limit: int = None, offset: int = None
    ) -> typing.List[OAuth2Client]:
        url = urljoin(api.url_, cls.url_)
        params = filter_none({"limit": limit, "offset": offset})
        response = api._request("GET", url, params=params)
        payload = response.json()
        return [OAuth2Client._from_dict(d, parent=api) for d in payload]

    @classmethod
    def create(
        cls,
        api: HydraAdmin,
        allowed_cors_origins: typing.List[str] = None,
        audience: typing.List[str] = None,
        backchannel_logout_session_required: bool = None,
        backchannel_logout_uri: str = None,
        client_id: str = None,
        client_name: str = None,
        client_secret: str = None,
        client_secret_expires_at: int = None,
        client_uri: str = None,
        contacts: typing.List[str] = None,
        frontchannel_logout_session_required: bool = None,
        frontchannel_logout_uri: str = None,
        grant_types: typing.List[str] = None,
        jwks: dict = None,
        jwks_uri: str = None,
        logo_uri: str = None,
        owner: str = None,
        policy_uri: str = None,
        post_logout_redirect_uris: typing.List[str] = None,
        redirect_uris: typing.List[str] = None,
        redirect_object_signing_alg: str = None,
        request_uris: typing.List[str] = None,
        response_uris: typing.List[str] = None,
        scope: str = None,
        sector_identifier_uri: str = None,
        subject_type: str = None,
        token_endpoint_auth_method: str = None,
        tos_uri: str = None,
        userinfo_signed_response_alg: str = None,
    ) -> OAuth2Client:
        url = urljoin(api.url_, cls.url_)
        data = filter_none(
            {
                "allowed_cors_origins": allowed_cors_origins,
                "audience": audience,
                "backchannel_logout_session_required": backchannel_logout_session_required,
                "backchannel_logout_uri": backchannel_logout_uri,
                "client_id": client_id,
                "client_name": client_name,
                "client_secret": client_secret,
                "client_secret_expires_at": client_secret_expires_at,
                "client_uri": client_uri,
                "contacts": contacts,
                "frontchannel_logout_session_required": frontchannel_logout_session_required,
                "frontchannel_logout_uri": frontchannel_logout_uri,
                "grant_types": grant_types,
                "jwks": jwks,
                "jwks_uri": jwks_uri,
                "logo_uri": logo_uri,
                "owner": owner,
                "policy_uri": policy_uri,
                "post_logout_redirect_uris": post_logout_redirect_uris,
                "redirect_uris": redirect_uris,
                "redirect_object_signing_alg": redirect_object_signing_alg,
                "request_uris": request_uris,
                "response_uris": response_uris,
                "scope": scope,
                "sector_identifier_uri": sector_identifier_uri,
                "subject_type": subject_type,
                "token_endpoint_auth_method": token_endpoint_auth_method,
                "tos_uri": tos_uri,
                "userinfo_signed_response_alg": userinfo_signed_response_alg,
            }
        )
        response = api._request("POST", url, json=data)
        return cls._from_dict(response.json(), parent=api)

    @classmethod
    def _get(cls, api: HydraAdmin, client_id: str) -> OAuth2Client:
        url = urljoin(api.url_, cls.url_, client_id)
        response = api._request("GET", url)
        return cls._from_dict(response.json(), parent=api)

    def update(
        self,
        allowed_cors_origins: typing.List[str] = None,
        audience: typing.List[str] = None,
        backchannel_logout_session_required: bool = None,
        backchannel_logout_uri: str = None,
        client_id: str = None,
        client_name: str = None,
        client_secret: str = None,
        client_secret_expires_at: int = None,
        client_uri: str = None,
        contacts: typing.List[str] = None,
        frontchannel_logout_session_required: bool = None,
        frontchannel_logout_uri: str = None,
        grant_types: typing.List[str] = None,
        jwks: dict = None,
        jwks_uri: str = None,
        logo_uri: str = None,
        owner: str = None,
        policy_uri: str = None,
        post_logout_redirect_uris: typing.List[str] = None,
        redirect_uris: typing.List[str] = None,
        redirect_object_signing_alg: str = None,
        request_uris: typing.List[str] = None,
        response_uris: typing.List[str] = None,
        scope: str = None,
        sector_identifier_uri: str = None,
        subject_type: str = None,
        token_endpoint_auth_method: str = None,
        tos_uri: str = None,
        userinfo_signed_response_alg: str = None,
    ) -> OAuth2Client:
        data = filter_none(
            {
                "allowed_cors_origins": allowed_cors_origins,
                "audience": audience,
                "backchannel_logout_session_required": backchannel_logout_session_required,
                "backchannel_logout_uri": backchannel_logout_uri,
                "client_id": client_id,
                "client_name": client_name,
                "client_secret": client_secret,
                "client_secret_expires_at": client_secret_expires_at,
                "client_uri": client_uri,
                "contacts": contacts,
                "frontchannel_logout_session_required": frontchannel_logout_session_required,
                "frontchannel_logout_uri": frontchannel_logout_uri,
                "grant_types": grant_types,
                "jwks": jwks,
                "jwks_uri": jwks_uri,
                "logo_uri": logo_uri,
                "owner": owner,
                "policy_uri": policy_uri,
                "post_logout_redirect_uris": post_logout_redirect_uris,
                "redirect_uris": redirect_uris,
                "redirect_object_signing_alg": redirect_object_signing_alg,
                "request_uris": request_uris,
                "response_uris": response_uris,
                "scope": scope,
                "sector_identifier_uri": sector_identifier_uri,
                "subject_type": subject_type,
                "token_endpoint_auth_method": token_endpoint_auth_method,
                "tos_uri": tos_uri,
                "userinfo_signed_response_alg": userinfo_signed_response_alg,
            }
        )
        response = self._request("PUT", self.url_, json=data)
        payload = response.json()
        # Create another instance, so all converters are run
        other = self._from_dict(payload, parent=self.parent_)
        self.__dict__.update(other.__dict__)
        return self

    def delete(self) -> None:
        self._request("DELETE", self.url_)



# -*- coding: utf-8 -*-
from distutils.core import setup

package_dir = \
{'': 'src'}

packages = \
['hydra_client']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=19.2,<20.0',
 'python-dateutil>=2.8,<3.0',
 'requests-oauthlib>=1.0,<2.0',
 'requests>=2.21,<3.0']

setup_kwargs = {
    'name': 'hydra-client',
    'version': '0.5.0',
    'description': 'Client library for ORY Hydra (OAuth2 and OpenID Connect provider)',
    'long_description': None,
    'author': 'Simon Westphahl',
    'author_email': 'westphahl@gmail.com',
    'url': 'https://github.com/westphahl/hydra-client',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from __future__ import annotations

import typing

import attr

from .common import OpenIDConnectContext
from .model import Entity, Resource
from .oauth2 import OAuth2Client
from .utils import filter_none, urljoin

if typing.TYPE_CHECKING:
    from .api import HydraAdmin


@attr.s(auto_attribs=True, kw_only=True)
class ConsentRequest(Resource):
    acr: str
    challenge: str
    client: OAuth2Client
    context: dict = attr.ib(factory=dict)
    login_challenge: str
    login_session_id: str
    oidc_context = OpenIDConnectContext
    request_url: str
    requested_access_token_audience: typing.List[str]
    requested_scope: typing.List[str]
    skip: bool
    subject: str

    url_ = "/oauth2/auth/requests/consent"

    def _post_bind(self) -> None:
        self.url_ = urljoin(self.parent_.url_, self.url_)

    @classmethod
    def _params(cls, challenge: str) -> dict:
        return {"consent_challenge": challenge}

    @classmethod
    def _get(cls, api: HydraAdmin, challenge: str) -> ConsentRequest:
        url = urljoin(api.url_, cls.url_)
        response = api._request("GET", url, params=cls._params(challenge))
        payload = response.json()
        return cls._from_dict(payload, parent=api)

    def accept(
        self,
        grant_access_token_audience: typing.Iterable[str] = None,
        grant_scope: typing.Iterable[str] = None,
        remember: bool = False,
        remember_for: int = None,
        session: dict = None,
    ) -> str:
        data = filter_none(
            {
                "grant_access_token_audience": grant_access_token_audience,
                "grant_scope": grant_scope,
                "remember": remember,
                "remember_for": remember_for,
                "session": session,
            }
        )
        url = urljoin(self.url_, "accept")
        response = self._request(
            "PUT", url, params=self._params(self.challenge), json=data
        )
        payload = response.json()
        return payload["redirect_to"]

    def reject(
        self,
        error: str = None,
        error_debug: str = None,
        error_description: str = None,
        error_hint: str = None,
        status_code: int = None,
    ) -> str:
        url = urljoin(self.url_, "reject")
        data = filter_none(
            {
                "error": error,
                "error_debug": error_debug,
                "error_description": error_description,
                "error_hint": error_hint,
                "status_code": status_code,
            }
        )
        response = self._request(
            "PUT", url, params=self._params(self.challenge), json=data
        )
        payload = response.json()
        return payload["redirect_to"]


@attr.s(auto_attribs=True, kw_only=True)
class ConsentRequestSession(Entity):
    access_token: dict
    id_token: dict


@attr.s(auto_attribs=True, kw_only=True)
class ConsentSession(Resource):
    consent_request: ConsentRequest
    grant_access_token_audience: typing.List[str]
    grant_scope: typing.List[str]
    remember: bool
    remember_for: int
    session: ConsentRequestSession

    url_ = "/oauth2/auth/sessions/consent"

    @classmethod
    def _params(cls, subject: str, client: str = None) -> dict:
        return filter_none({"subject": subject, "client": client})

    @classmethod
    def _list(cls, api: HydraAdmin, subject: str) -> typing.Iterator[ConsentSession]:
        url = urljoin(api.url_, cls.url_)
        response = api._request("GET", url, params=cls._params(subject))
        session_list = response.json()
        for consent_session in session_list:
            yield ConsentSession._from_dict(consent_session, parent=api)

    @classmethod
    def _revoke(
        cls, api: HydraAdmin, subject: str, client: typing.Optional[str]
    ) -> None:
        url = urljoin(api.url_, cls.url_)
        # This returns 204/201 without any content
        api._request("DELETE", url, params=cls._params(subject, client))



from __future__ import annotations

import collections
import typing

import attr
import requests

from . import exceptions

T = typing.TypeVar("T", bound="Entity")
U = typing.TypeVar("U", bound="Resource")


class Entity:
    @classmethod
    def _from_dict(cls: typing.Type[T], data: dict) -> T:
        fields = attr.fields_dict(cls)
        clean_data = {k: v for k, v in data.items() if k in fields}
        return cls(**clean_data)  # type: ignore


class Resource(Entity):
    url_: str

    def _request(
        self, method: str, url: str, params: dict = None, json: dict = None
    ) -> requests.Response:
        try:
            response = typing.cast(
                requests.Response,
                self.session_.request(method, url, params=params, json=json),
            )
        except (
            requests.exceptions.ConnectionError,
            requests.exceptions.Timeout,
        ) as exc:
            raise exceptions.ConnectionError from exc
        except requests.exceptions.RequestException as exc:
            raise exceptions.TransportError from exc
        except AttributeError:
            raise exceptions.UnboundResourceError

        try:
            response.raise_for_status()
        except requests.exceptions.HTTPError as exc:
            wrapper_exc = exceptions.status_map.get(
                exc.response.status_code, exceptions.HTTPError
            )
            raise wrapper_exc from exc
        return response

    def _post_bind(self) -> None:
        pass

    def _bind(self, parent: Resource) -> None:
        self.parent_ = parent
        self.session_ = getattr(self.parent_, "session_", None)
        self._post_bind()
        for field in attr.fields(self.__class__):
            value = getattr(self, field.name)
            try:
                value._bind(self)
            except AttributeError:
                pass

    @classmethod
    def _from_dict(cls: typing.Type[U], data: dict, parent: Resource = None) -> U:
        instance = super()._from_dict(data)
        if parent is not None:
            instance._bind(parent)
        return instance


class ResourceList(collections.UserList):
    def _bind(self, parent: Resource) -> None:
        for item in self:
            try:
                item._bind(parent)
            except AttributeError:
                pass


def list_attr(klass: typing.Type[Entity]) -> typing.Any:
    def converter(entity_list: typing.List[dict]) -> ResourceList:
        if entity_list is None:
            entity_list = []
        return ResourceList(klass._from_dict(d) for d in entity_list)

    return attr.ib(converter=converter)
